/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.contabilidad.FacadeConta;

import com.contabilidad.EntConta.HorasDeTrabajoTemporal;
import com.contabilidad.EntConta.SalariosEventuale;
import com.contabilidad.EntConta.VCalculohorassumas;

import javax.ejb.Local;
import java.util.Date;
import java.util.List;

/**
 *
 * @author obper
 */
@Local
public interface HorasDeTrabajoTemporalFacadeLocal {

    void create(HorasDeTrabajoTemporal horasDeTrabajoTemporal);

    void edit(HorasDeTrabajoTemporal horasDeTrabajoTemporal);

    void remove(HorasDeTrabajoTemporal horasDeTrabajoTemporal);

    HorasDeTrabajoTemporal find(Object id);

    List<HorasDeTrabajoTemporal> findAll();

    List<HorasDeTrabajoTemporal> findRange(int[] range);

    int count();

    List<HorasDeTrabajoTemporal> obtnerHoras(String codigoEmpleado);

    void ingresarHora(String codigoEmpleado, Date fechaIni, Date fechaFin, boolean corrido);

    List<VCalculohorassumas> obtnerHorasCal(String codigoEmpleado, Date fecha);

    void borrarHora(String codigoEmpleado);
    
    List<HorasDeTrabajoTemporal>  obtnerHorasIdPag(String codiPag);


    List<VCalculohorassumas> obtnerHorasCalEx(String codigoEmpleado);

    SalariosEventuale busq();

}
